import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, Empty, message, Row, Col, Pagination, Switch, notification } from 'antd'
import { DownOutlined } from '@ant-design/icons'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import preImg from '@/assets/img/prev.png'
import nextImg from '@/assets/img/next.png'
import MKEmitter from '@/utils/events.js'
import TimerTask from '@/utils/timer-task.js'
import asyncComponent from '@/utils/asyncComponent'
import './index.scss'

const CardItem = asyncComponent(() => import('../cardItem'))
const TableHeader = asyncComponent(() => import('../tableHeader'))
const MainAction = asyncComponent(() => import('@/tabviews/zshare/actionList'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class DataCard extends Component {
  static propTpyes = {
    config: PropTypes.object
  }

  state = {
    BID: '',                   // 上级ID
    BData: '',                 // 上级行数据
    config: null,              // 图表配置信息
    search: null,              // 搜索条件
    pageIndex: 1,              // 页码
    pageSize: 10,
    orderBy: '',
    pageOptions: [],
    activeKey: '',
    selectKeys: [],
    selectedData: [],
    loading: false,
    card: null,
    data: [],
    total: 0,
    precards: [],
    nextcards: [],
    selected: 'false',
    pickup: false
  }

  loaded = false
  supModules = []

  UNSAFE_componentWillMount () {
    const { config } = this.props

    let _config = fromJS(config).toJS()
    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    let _card = null
    let precards = []
    let nextcards = []

    if (_config.wrap.controlField) {
      if (_config.wrap.controlVal) {
        _config.wrap.controlVal = _config.wrap.controlVal.split(',')
      } else {
        _config.wrap.controlVal = ['']
      }
    }

    _config.$extend = false
    _config.$empty = true

    _config.subcards.forEach(item => {
      if (item.setting.click === 'button' && !item.setting.linkbtn) {
        item.elements.forEach(ele => {
          if (ele.eleType === 'button') {
            item.setting.linkbtn = ele.uuid
          }
        })
        if (!item.setting.linkbtn) {
          item.setting.click = ''
        }
      }

      if (item.style.clear === 'left') {
        item.wStyle = {clear: 'left'}
      } else if (item.style.clear === 'right') {
        item.wStyle = {float: 'right'}
      }

      delete item.style.clear

      if (item.$cardType !== 'extendCard') {
        _card = item
      } else if (!_card) {
        _config.$extend = true
        if (item.setting.width !== 24) {
          _config.$empty = false
        }
        precards.push(item)
      } else {
        _config.$extend = true
        _config.$empty = false
        nextcards.push(item)
      }
    })

    _config.subcards = null

    let selected = 'false'
    if (_config.wrap.selected === 'always' || _config.wrap.selected === 'init' || _config.wrap.selected === 'sign') {
      selected = _config.wrap.selected
    } else {
      _config.wrap.selected = 'false'
    }

    _config.wrap.selStyle = _config.wrap.selStyle || 'active'
    _config.wrap.pagestyle = _config.wrap.pagestyle || 'page'
    _config.wrap.scale = _config.wrap.scale === 'true' ? 'scale' : ''
    _config.wrap.layout = (_config.wrap.layout || 'grid') + '-layout float-' + (_config.wrap.cardFloat || 'left')

    _config.wrap.wrapClass = `${_config.wrap.selStyle} ${_config.wrap.cardType || ''} ${_config.wrap.scale} ${config.wrap.hover === 'true' ? 'mk-hover' : ''}`

    if (_config.wrap.shifting === 'true') {
      _config.wrap.shifting = 'shifting'
    } else {
      _config.wrap.shifting = ''
    }

    let pageOptions = ['10', '25', '50', '100', '500', '1000']

    if (!_config.setting.laypage) {
      _config.wrap.pagestyle = 'none'
    }

    if (_config.wrap.pagestyle === 'page') {
      let size = (_config.setting.pageSize || 10) + ''
      if (!pageOptions.includes(size)) {
        pageOptions.push(size)
        pageOptions = pageOptions.sort((a, b) => a - b)
      }

      if (_config.wrap.maxPageSize) {
        pageOptions = pageOptions.filter(item => item <= _config.wrap.maxPageSize)
      }
    }

    let _data = []
    if (_config.setting.sync === 'true') {
      _config.setting.onload = 'false'

      if (window.GLOB.SyncData.has(_config.dataName)) {
        _data = window.GLOB.SyncData.get(_config.dataName) || []
  
        if (_config.$cache) {
          Api.writeCacheConfig(_config.uuid, fromJS(_data).toJS(), BID)
        }
  
        _config.setting.sync = 'false'
  
        _data = _data.map((item, index) => {
          item.key = index
          item.$$uuid = item[_config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + 1 + ''
  
          if (_config.wrap.controlField) {
            if (_config.wrap.controlVal.includes(item[_config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          return item
        })
  
        this.loaded = true
  
        window.GLOB.SyncData.delete(_config.dataName)
      }
    } else if (_config.setting.supModule && !BID && _config.wrap.supKey !== 'false') {
      _config.setting.onload = 'false'
    }

    if (_config.wrap.zHeight) {
      if (_config.wrap.zHeight <= 100) {
        if (_config.wrap.zHeight < 0) {
          _config.wrap.zHeight = `calc(100vh - ${-_config.wrap.zHeight}px)`
        } else {
          _config.wrap.zHeight = _config.wrap.zHeight + 'vh'
        }
      }

      if (_config.style) {
        delete _config.style.height
      }
      _config.wrap.layout += ' fix-height'
    } else {
      _config.wrap.zHeight = 'none'
    }

    this.setState({
      pageSize: _config.setting.pageSize || 10,
      pageOptions,
      selected,
      precards,
      nextcards,
      data: _data,
      BID: BID || '',
      BData: BData || '',
      config: _config,
      card: _card,
      search: _config.$searches
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('mkCheckAll', this.mkCheckAll)
    MKEmitter.addListener('refreshLineData', this.refreshLineData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.setting.useMSearch) {
      MKEmitter.addListener('searchRefresh', this.searchRefresh)
    }

    if (config.setting.sync === 'true') {
      MKEmitter.addListener('transferSyncData', this.transferSyncData)
    }
    
    if (config.timer) {
      this.timer = new TimerTask()
      this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {
        this.setState({
          pageIndex: 1
        }, () => {
          this.loadData('', 'timer')
        })
      })
    }

    this.initExec()

    this.autoExec()
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkCheckAll', this.mkCheckAll)
    MKEmitter.removeListener('searchRefresh', this.searchRefresh)
    MKEmitter.removeListener('refreshLineData', this.refreshLineData)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  initExec = () => {
    const { config, BID } = this.state

    this.loaded && this.prevCheck()

    if (config.$cache) {
      if (config.$time && !config.setting.laypage) {
        if (!this.loaded) {
          Api.getLCacheConfig(config.uuid, config.$time, BID).then(res => {
            if (!res.valid && config.setting.onload === 'true') {
              setTimeout(() => {
                this.loadData('', 'init')
              }, config.setting.delay || 0)
            }

            if (!res.data || this.loaded) return

            let _data = res.data.map((item, index) => {
              item.key = index
              item.$$uuid = item[config.setting.primaryKey] || ''
              item.$Index = index + 1 + ''
    
              if (config.wrap.controlField) {
                if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
                  item.$disabled = true
                }
              }
    
              return item
            })
    
            this.setState({data: _data}, () => {
              this.prevCheck()
            })
          })
        }
      } else {
        if (!this.loaded) {
          Api.getLCacheConfig(config.uuid, 0, BID).then(res => {
            if (!res.data || this.loaded) return
    
            let _data = res.data.map((item, index) => {
              item.key = index
              item.$$uuid = item[config.setting.primaryKey] || ''
              item.$Index = index + 1 + ''
    
              if (config.wrap.controlField) {
                if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
                  item.$disabled = true
                }
              }
    
              return item
            })
    
            this.setState({data: _data})
          })
        }

        if (config.setting.onload === 'true') {
          setTimeout(() => {
            this.loadData('', 'init')
          }, config.setting.delay || 0)
        }
      }
    } else if (config.setting.onload === 'true') {
      setTimeout(() => {
        this.loadData()
      }, config.setting.delay || 0)
    }
  }

  autoExec = (times) => {
    const { config } = this.state

    if (!config.wrap.autoExec) return

    let btn = document.getElementById('button' + config.wrap.autoExec)

    this.autoTimer && clearTimeout(this.autoTimer)

    if (btn) {
      MKEmitter.emit('triggerBtnId', config.wrap.autoExec, [])
    } else if (!times || times < 20) {
      times = times ? times + 1 : 1
      this.autoTimer = setTimeout(() => {
        this.autoExec(times)
      }, 1000)
    }
  }

  transferSyncData = (syncId) => {
    const { config } = this.state

    if (config.$syncId !== syncId) return

    const { BID, BData } = this.state

    let _data = window.GLOB.SyncData.get(config.dataName) || []

    if (config.$cache) {
      Api.writeCacheConfig(config.uuid, fromJS(_data).toJS(), BID)
    }

    _data = _data.map((item, index) => {
      item.key = index
      item.$$uuid = item[config.setting.primaryKey] || ''
      item.$$BID = BID || ''
      item.$$BData = BData || ''
      item.$Index = index + 1 + ''

      if (config.wrap.controlField) {
        if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
          item.$disabled = true
        }
      }

      return item
    })

    this.prevCheck()

    this.loaded = true

    this.setState({data: _data})

    window.GLOB.SyncData.delete(config.dataName)

    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
  }

  searchRefresh = (searchId) => {
    const { config } = this.state

    if (config.$searchId !== searchId) return
    
    this.reloadIndex()
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id = '', lines) => {
    const { config, BID } = this.state

    if (config.uuid !== menuId) return

    let _id = config.wrap.selected === 'always' ? id : ''
    if (config.supNodes) {
      if (['mainline', 'maingrid', 'popclose'].includes(position)) {
        let supNode = this.supModules[this.supModules.length - 1]
        config.supNodes.forEach((item, i) => {
          setTimeout(() => {
            if (supNode && supNode.key === item) {
              MKEmitter.emit('reloadData', item, supNode.value)
            } else {
              MKEmitter.emit('reloadData', item)
            }
          }, i * 10)
        })
      } else {
        if (position === 'line' || position === 'line_grid') {
          if (lines && lines.length === 1) {
            if (config.forbidLine) {
              this.loadData(lines[0].$$uuid, 'repage')
            } else {
              this.loadLinedata(lines[0].$$uuid, position)
            }
          } else {
            this.loadData(_id, 'repage')
          }
        } else if (btn.resetPageIndex !== 'false') {
          this.reloadIndex()
        } else {
          this.loadData(_id, 'repage')
        }
      }
    } else {
      let supModule = config.setting.supModule

      if (position === 'line' || position === 'line_grid') {
        if (lines && lines.length === 1) {
          if (config.forbidLine) {
            this.loadData(lines[0].$$uuid, 'repage')
          } else {
            this.loadLinedata(lines[0].$$uuid, position)
          }
        } else {
          this.loadData(_id, 'repage')
        }
      } else if (['mainline', 'maingrid', 'popclose'].includes(position) && supModule) {
        MKEmitter.emit('reloadData', supModule, position === 'maingrid' ? '' : BID)
      } else if (btn.resetPageIndex !== 'false') {
        this.reloadIndex()
      } else {
        this.loadData(_id, 'repage')
      }
    }
  }

  refreshLineData = (menuId, btn, uuid, count) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return
    
    let _data = fromJS(data).toJS().map(item => {
      if (item.$$uuid === uuid) {
        item[btn.field] = count
      }
      return item
    })

    this.setState({
      data: _data
    })
  }

  reloadIndex = () => {
    this.setState({
      pageIndex: 1
    }, () => {
      this.loadData()
    })
  }

  prevCheck = (id) => {
    const { selected } = this.state

    if (selected === 'false' && !id) return

    setTimeout(() => {
      this.checkTopLine(id, selected)
    }, 10)

    if (selected === 'init') {
      this.setState({selected: 'false'})
    }
  }

  checkTopLine = (id, selected) => {
    const { config, data } = this.state

    let index = -1
    let keys = []
    let items = []

    if (data.length > 0) {
      if (id) {
        index = data.findIndex(item => item.$$uuid === id && !item.$disabled)
      }
  
      if (index !== -1) {
        keys = [index]
        items = [data[index]]
      } else if (selected === 'sign') {
        data.forEach((item, i) => {
          if (!item.$disabled && item.selected === 'true') {
            items.push(item)
            keys.push(i)
            index = i
          }
        })
      } else if (selected !== 'false') {
        if (!data[0].$disabled) {
          index = 0
          keys = [index]
          items = [data[index]]
        }
      }
    }

    if (index === -1) {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
      return
    }

    let item = items[items.length - 1]

    this.setState({
      activeKey: index,
      selectKeys: keys,
      selectedData: items
    })

    MKEmitter.emit('resetSelectLine', config.uuid, item.$$uuid, item)
    if (config.setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', config.uuid, items, data.length === keys.length)
    }
  }

  mkCheckAll = (menuId, checked) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return

    if (checked) {
      let index = ''
      let keys = []
      let items = []
      let last = ''

      data.forEach((item, i) => {
        if (item.$disabled) return

        items.push(item)
        keys.push(i)
        index = i
        last = item
      })

      this.setState({
        activeKey: index,
        selectKeys: keys,
        selectedData: items
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, last ? last.$$uuid : '', last)
      MKEmitter.emit('syncBalconyData', config.uuid, items, data.length > 0 && data.length === keys.length)
      if (items.length === 0) {
        message.warning(window.GLOB.dict['no_data'] || '未获取到数据！')
      }
    } else {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      MKEmitter.emit('syncBalconyData', config.uuid, [], false)
    }
  }

  checkAll = () => {
    const { config, data, selectedData } = this.state

    if (config.wrap.cardType !== 'checkbox') return
    if (data.length === 0) return
    
    if (selectedData.length === 0 || selectedData.length < data.length) {
      let index = ''
      let keys = []
      let items = []
      let last = ''

      data.forEach((item, i) => {
        if (item.$disabled) return

        items.push(item)
        keys.push(i)
        index = i
        last = item
      })

      this.setState({
        activeKey: index,
        selectKeys: keys,
        selectedData: items
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, last ? last.$$uuid : '', last)
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, items, data.length === keys.length)
      }
    } else {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
    }
  }

  reloadData = (menuId, id) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    if (!id) {
      this.reloadIndex()
    } else if (config.forbidLine) {
      this.loadData(id, 'repage')
    } else {
      this.loadLinedata(id)
    }
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (config.supNodes) {
      if (!config.supNodes.includes(MenuID)) return
      this.supModules = this.supModules.filter(item => item.key !== MenuID)
      let bid = ''
      let _data = null

      if (id) {
        this.supModules.push({key: MenuID, value: id, data})
      }
      if (this.supModules.length > 0) {
        bid = this.supModules[this.supModules.length - 1].value
        _data = this.supModules[this.supModules.length - 1].data
      }

      if (bid !== this.state.BID || bid !== '') {
        this.setState({ BID: bid, BData: _data, pageIndex: 1 }, () => {
          this.loadData()
        })
      }
    } else {
      if (!config.setting.supModule || config.setting.supModule !== MenuID) return
      if (id !== this.state.BID || id !== '') {
        this.setState({ BID: id, BData: data, pageIndex: 1 }, () => {
          if (!config.checkBid) {
            this.loadData()
          }
        })
      }
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { config, search, orderBy } = this.state

    if (config.uuid !== menuId) return

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    callback({
      orderBy: orderBy || config.setting.order || '',
      search: searches
    })
  }

  async loadData (id, type) {
    const { config, pageIndex, pageSize, search, BID, BData, selected, orderBy } = this.state

    if (config.setting.supModule && !BID && config.wrap.supKey !== 'false') { // BID 不存在时，不做查询
      this.loaded = true
      this.requestId = ''

      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: [],
        pageIndex: 1,
        data: [],
        total: 0,
        loading: false
      })
      
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
      if (config.$hasTopModule) {
        window.GLOB.CacheData.set(config.uuid + 'tb', { $$empty: true, $$uuid: '' })
        MKEmitter.emit('mkPublicData', config.uuid + 'tb', { $$empty: true, $$uuid: '' })
      }
      return
    }

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    if (config.$s_req && searches.filter(item => item.required && item.value === '').length > 0) {
      return
    }

    if (type !== 'timer') {
      this.setState({
        loading: true
      })
    }

    let _orderBy = orderBy || config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, searches, _orderBy, pageIndex, pageSize, BID)

    this.requestId = config.uuid + new Date().getTime()

    let result = await Api.genericInterface(param, '', '', this.requestId)
    if (result.status) {
      if (result.$requestId && this.requestId !== result.$requestId) return

      if (type === 'repage' && result.data.length === 0 && pageIndex > 1) {
        this.reloadIndex()
        return
      }

      let start = 1
      if (config.setting.laypage) {
        start = pageSize * (pageIndex - 1) + 1
      }

      this.loaded = true
      if (config.$cache && type === 'init') {
        Api.writeCacheConfig(config.uuid, result.data || [], BID)
      }

      if (selected !== 'false' || id) {
        this.prevCheck(id)
      } else {
        MKEmitter.emit('resetSelectLine', config.uuid, '', '')
        if (config.setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, [], false)
        }
      }

      let data = []

      if (type === 'plus') {
        let _data = (this.state.data || []).concat(result.data || [])
        data = _data.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + 1 + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          return item
        })
      } else {
        data = result.data.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + start + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          return item
        })
      }

      let total = result.total || 0
      if (config.setting.custompage && data.length) {
        total = data[data.length - 1].mk_total || 0
      }

      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: [],
        data: data,
        total: total,
        pickup: false,
        loading: false
      })

      if (config.$hasTopModule) {
        window.GLOB.CacheData.set(config.uuid + 'tb', data[0] || { $$empty: true, $$uuid: '' })
        MKEmitter.emit('mkPublicData', config.uuid + 'tb', data[0] || { $$empty: true, $$uuid: '' })
      }

      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }

      UtilsDM.querySuccess(result)
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()

      UtilsDM.queryFail(result)
    }
  }

  /**
   * @description 获取单行数据
   */ 
  async loadLinedata (id, position) {
    const { config, pageIndex, pageSize, search, BID, BData, orderBy } = this.state

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    this.setState({
      loading: true
    })

    let _orderBy = orderBy || config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, searches, _orderBy, pageIndex, pageSize, BID, id)

    let result = await Api.genericInterface(param)
    if (result.status) {
      if (position === 'line_grid' && (!result.data || !result.data[0])) {
        this.reloadIndex()
        return
      }

      let data = fromJS(this.state.data).toJS()
      let selectedData = fromJS(this.state.selectedData).toJS()
      let selectKeys = fromJS(this.state.selectKeys).toJS()
      let activeKey = this.state.activeKey

      if (result.data && result.data[0]) {
        let _data = result.data[0]
        _data.$$uuid = _data[config.setting.primaryKey] || ''
        _data.$$BID = BID || ''
        _data.$$BData = BData || ''

        if (config.wrap.controlField) {
          if (config.wrap.controlVal.includes(_data[config.wrap.controlField] + '')) {
            _data.$disabled = true
          }
        }

        data = data.map(item => {
          if (item.$$uuid === _data.$$uuid) {
            _data.key = item.key
            _data.$Index = item.$Index
            return _data
          }
          return item
        })

        if (!_data.$Index) {
          this.setState({
            loading: false
          })
          return
        }

        selectedData = selectedData.map(item => {
          if (item.$$uuid === _data.$$uuid) {
            return _data
          }
          return item
        })

        this.setState({
          data: data,
          selectedData: selectedData,
          loading: false
        })

        if (_data.key === activeKey) {
          MKEmitter.emit('resetSelectLine', config.uuid, _data.$$uuid || '', _data)
        }
        if (config.setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, selectedData, data.length === selectedData.length)
        }
      } else {
        let index = data.findIndex(item => item.$$uuid === id)

        if (index === -1) {
          this.setState({
            loading: false
          })
          return
        }

        data = data.filter(item => item.$$uuid !== id)
        data = data.map((item, i) => {
          item.key = i
          return item
        })
        
        let trans = activeKey === index
        let _item = null
        
        if (config.wrap.cardType) {
          selectKeys = selectKeys.filter(key => key !== index)
          selectKeys = selectKeys.map(key => key > index ? key - 1 : key)

          selectedData = selectKeys.map(key => data[key]).filter(Boolean)

          activeKey = selectKeys.length ? selectKeys[selectKeys.length - 1] : ''

          if (trans && selectedData.length) {
            _item = selectedData[selectedData.length - 1]
          }
        }

        this.setState({
          data: data,
          activeKey: activeKey,
          selectKeys: selectKeys,
          selectedData: selectedData,
          loading: false
        })

        if (trans) {
          MKEmitter.emit('resetSelectLine', config.uuid, (_item ? _item.$$uuid : ''), _item)
        }

        if (config.setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, selectedData, data.length === selectedData.length)
        }
      }
    } else {
      this.setState({
        loading: false
      })
      
      UtilsDM.queryFail(result)
    }
  }

  loadMore = () => {
    const { total, pageIndex, pageSize, loading } = this.state

    if (loading || pageSize * pageIndex >= total) {
      return
    }

    this.setState({
      pageIndex: pageIndex + 1
    }, () => {
      this.loadData('', 'plus')
    })
  }

  prevPage = () => {
    const { pageIndex } = this.state

    if (pageIndex === 1) return

    this.setState({
      pageIndex: pageIndex - 1
    }, () => {
      this.loadData()
    })
  }

  nextPage = () => {
    const { pageIndex, pageSize, total } = this.state
    let _total = pageSize * pageIndex

    if (_total >= total) return

    this.setState({
      pageIndex: pageIndex + 1
    }, () => {
      this.loadData()
    })
  }

  changePageIndex = (page) => {
    this.setState({
      pageIndex: page
    }, () => {
      this.loadData()
    })
  }

  refreshSearch = (list) => {
    const { config, BID } = this.state

    if (config.setting.supModule && !BID && config.wrap.supKey !== 'false') {
      notification.warning({
        top: 92,
        message: window.GLOB.dict['sup_key_req'] || '需要上级主键值！',
        duration: 5
      })
      this.setState({
        search: list
      })
      return
    }

    this.setState({
      search: list,
      pageIndex: 1
    }, () => {
      this.loadData()
    })
  }

  pageSizeChange = (current, size) => {
    this.setState({
      pageIndex: current,
      pageSize: size
    }, () => {
      this.loadData()
    })
  }
  
  changeCard = (index, item) => {
    const { config, selectKeys, selectedData, activeKey, data } = this.state

    if (!config.wrap.cardType) return
    if (item.$disabled) return
    
    let _selectKeys = []
    let _selectedData = []
    let _activeKey = ''
    let _item = item
    if (config.wrap.cardType === 'checkbox') {
      if (activeKey === index) {
        _selectKeys = selectKeys.filter(key => key !== index)
        _selectedData = selectedData.filter(cell => cell.key !== index)
        _activeKey = _selectKeys.slice(-1)[0]
        _item = _selectedData.slice(-1)[0] || ''
      } else if (selectKeys.indexOf(index) > -1) {
        _selectKeys = selectKeys.filter(key => key !== index)
        _selectedData = selectedData.filter(cell => cell.key !== index)
        _activeKey = activeKey
        _item = _selectedData.filter(cell => cell.key === activeKey)[0] || ''
      } else {
        _selectKeys = [...selectKeys, index]
        _selectedData = [...selectedData, item]
        _activeKey = index
      }
    } else {
      if (activeKey === index) return

      _selectedData = [item]
      _selectKeys = [index]
      _activeKey = index
    }

    this.setState({
      activeKey: _activeKey,
      selectKeys: _selectKeys,
      selectedData: _selectedData
    })

    MKEmitter.emit('resetSelectLine', config.uuid, (_item ? _item.$$uuid : ''), _item)
    if (config.setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', config.uuid, _selectedData, data.length === _selectedData.length)
    }
  }

  refreshByHeader = (sorter) => {
    this.setState({
      orderBy: sorter || ''
    }, () => {
      this.loadData()
    })
  }

  pickupChange = () => {
    this.setState({
      pickup: !this.state.pickup
    })
  }

  render() {
    const { config, precards, nextcards, loading, pageIndex, pageSize, total, card, activeKey, BID, BData, selectedData, selectKeys, pickup } = this.state

    let data = this.state.data

    if (config.wrap.empty === 'hidden' && data.length === 0) return null

    let _total = 0
    let switchable = false
    if (config.wrap.pagestyle === 'switch' && total > pageSize) {
      _total = pageSize * pageIndex
      switchable = true
    }

    let extendData = null

    if (config.$extend) {
      extendData = {$$BID: BID, $$BData: BData, $$selectedData: selectedData, $$type: 'extendCard'}
      
      if (data[0]) {
        if (selectedData[0]) {
          extendData = {...extendData, ...selectedData[0]}
        } else {
          extendData = {...extendData, ...data[0]}
        }
      }
    }

    let checkAll = ''
    if (config.wrap.selStyle && config.wrap.selStyle.indexOf('check') > -1) {
      if (selectedData.length > 0) {
        checkAll = selectedData.length < data.length ? ' half' : ' whole'
      }
    }

    if (pickup) {
      data = data.filter(item => selectKeys.includes(item.key))
    }

    return (
      <div className={'custom-data-card-box ' + config.wrap.shifting} id={'anchor' + config.uuid} style={config.style}>
        {loading ?
          <div className="loading-mask">
            {data.length ? <div className="ant-spin-blur"></div> : null}
            <Spin />
          </div> : null
        }
        <NormalHeader config={config} BID={BID} refresh={this.refreshSearch} />
        {config.action && config.action.length > 0 ?
          <MainAction
            BID={BID}
            BData={BData}
            setting={config.setting}
            actions={config.action}
            columns={config.columns}
            selectedData={selectedData}
          /> : null
        }
        {config.wrap.pickup === 'true' && this.state.data.length > 0 ? <div className="pickup-wrap"><Switch title="收起" checkedChildren={window.GLOB.dict['open'] || '开'} unCheckedChildren={window.GLOB.dict['shut'] || '关'} checked={pickup} onChange={this.pickupChange} /></div> : null}
        <div className={`data-zoom ${config.wrap.wrapClass}`}>
          {switchable ? <div className={'prev-page ' + (pageIndex === 1 ? 'disabled' : '')} onClick={this.prevPage}><div><div><img src={preImg} alt=""/></div></div></div> : null}
          <Row className={'card-row-list ' + config.wrap.layout} style={{maxHeight: config.wrap.zHeight}}>
            {precards.map((item, index) => (
              <Col key={'pre' + index} className="extend-card" style={item.wStyle} span={item.setting.width || 6}>
                {item.setting.cardRole === 'header' ? <TableHeader card={item} data={extendData} refresh={this.refreshByHeader}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </TableHeader> : <CardItem card={item} cards={config} data={extendData}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </CardItem>}
              </Col>
            ))}
            {data.map((item, index) => {
              let className = 'mk-card '
              if (config.wrap.parity === 'true') {
                if (index % 2 === 1) {
                  className += 'mk-even-line '
                }
              }
              if (item.$disabled) {
                className = 'mk-disabled'
              } else if (activeKey === item.key) {
                className += 'active'
              } else if (selectKeys.indexOf(item.key) > -1) {
                className += 'selected'
              }

              return (
                <Col className={className} key={index} style={card.wStyle} span={card.setting.width}>
                  <CardItem card={card} cards={config} data={item} onClick={() => {this.changeCard(item.key, item)}}>
                    <span className="circle-select"></span>
                  </CardItem>
                </Col>
              )
            })}
            {nextcards.map((item, index) => (
              <Col key={'next' + index} className="extend-card" style={item.wStyle} span={item.setting.width || 6}>
                {item.setting.cardRole === 'header' ? <TableHeader card={item} data={extendData} refresh={this.refreshByHeader}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </TableHeader> : <CardItem card={item} cards={config} data={extendData}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </CardItem>}
              </Col>
            ))}
          </Row>
          {switchable ? <div className={'prev-page ' + (total <= _total ? 'disabled' : '')} onClick={this.nextPage}><div><div><img src={nextImg} alt=""/></div></div></div> : null}
        </div>
        {config.$empty && data.length === 0 ? <Empty description={false}/> : null}
        {config.wrap.pagestyle === 'page' ? <Pagination size="small" total={total} showTotal={(t, range) => total > 0 ? `${range[0]}-${range[1]} ${window.GLOB.dict['of'] || '共'} ${total} ${window.GLOB.dict['items'] || '条'}` : `${total} ${window.GLOB.dict['items'] || '条'}`} pageSize={pageSize} showSizeChanger={true} pageSizeOptions={this.state.pageOptions} onChange={this.changePageIndex} onShowSizeChange={this.pageSizeChange} current={pageIndex}/> : null}
        {config.wrap.pagestyle === 'more' && data.length > 0 ? <div className={'mk-more' + (pageSize * pageIndex >= total ? ' disabled' : '')} onClick={this.loadMore}>{window.GLOB.dict['view_more'] || '查看更多'}<DownOutlined/></div> : null}
      </div>
    )
  }
}

export default DataCard